"""
VgDisplay - command ``vgdisplay``
=================================

"""

from .. import parser, CommandParser

import re
from insights.specs import Specs


@parser(Specs.vgdisplay)
class VgDisplay(CommandParser):
    """
    Parse the output of the ``vgdisplay -vv`` or ``vgdisplay`` commands.

    The ``vg_list`` property is the main access to the list of volume groups
    in the command output.  Each volume group is stored as a dictionary of
    keys and values drawn from the property list of the volume group.  The
    volume group's logical and physical volumes are stored in the 'Logical
    Volumes' and 'Physical Volumes' sub-keys, respectively.

    Sample command output of ``vgdisplay -vv`` (pruned for clarity)::

        Couldn't find device with uuid VVLmw8-e2AA-ECfW-wDPl-Vnaa-0wW1-utv7tV.

        --- Volume group ---
        VG Name               RHEL7CSB
        System ID
        Format                lvm2
        Metadata Areas        1
        Metadata Sequence No  13
        ...

        --- Logical volume ---
        LV Path                /dev/RHEL7CSB/Root
        LV Name                Root
        VG Name                RHEL7CSB
        LV Size                29.30 GiB
        ...

        --- Physical volumes ---
        PV Name               /dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194
        PV UUID               EfWV9V-03CX-E6zc-JkMw-yQae-wdzp-Je1KUn
        PV Status             allocatable
        Total PE / Free PE    118466 / 4036

    Volume groups are kept in the ``vg_list`` property in the order they were
    found in the file.

    Lines containing 'Couldn't find device with uuid' and 'missing physical
    volume' are stored in a ``debug_info`` property.

    Examples:
        >>> vg_info = shared[VgDisplay]
        >>> len(vg_info.vg_list)
        1
        >>> vgdata = vg_info.vg_list[0]
        >>> vgdata['VG Name']
        'RHEL7CSB'
        >>> vgdata['VG Size']
        '462.76 GiB'
        >>> 'Logical Volumes' in vgdata
        True
        >>> lvs = vgdata['Logical Volumes']
        >>> type(lvs)
        dict
        >>> lvs.keys()  # Note - keyed by device name
        ['/dev/RHEL7CSB/Root']
        >>> lvs['/dev/RHEL7CSB/Root']['LV Name']
        'Root'
        >>> lvs['/dev/RHEL7CSB/Root']['LV Size']
        '29.30 GiB'
        >>> 'Physical Volumes' in vgdata
        True
        >>> vgdata['Physical Volumes'].keys()
        ['/dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194']
        >>> vgdata['Physical Volumes']['/dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194']['PV UUID']
        'EfWV9V-03CX-E6zc-JkMw-yQae-wdzp-Je1KUn'
        >>> vg_info.debug_info
        ["Couldn't find device with uuid VVLmw8-e2AA-ECfW-wDPl-Vnaa-0wW1-utv7tV."]
    """

    _FILTER_INFO = [
        "Couldn't find device with uuid",
        "physical volumes missing",
    ]

    def parse_content(self, content):
        self.vg_list = []
        self.debug_info = []

        # Each section starts with a '(VG|LV|PV) Name' header.  This then
        # sets the amount of space between the start of the key and the start
        # of the value.  Unfortunately, there may be only one space between
        # the key and the value and there are spaces in the key, so it's
        # impossible to tell where the key stops and the value ends on space
        # alone.  However, the header sets the left-most column the value
        # starts at, so once we pick up the header we ignore the regex match
        # and get the key and value as substrings.
        header_re = re.compile(r'^(?P<key>VG Name|LV Path|PV Name)\s+(?P<val>\S.*)$')
        # To save different handling, we use this to refer to the current
        # dictionary we're storing data in, whether it be a VG, LV or PV.
        current_data_store = None
        value_start_column = 0
        # Each section of key-value data starts with the header as above and
        # ends with a blank line.  This prevents parsing of random error
        # and information messages as key-value pairs.
        in_keyval_section = False
        for line in content:
            line = line.strip()
            if any(debug in line for debug in self._FILTER_INFO):
                if line not in self.debug_info:
                    self.debug_info.append(line)
                in_keyval_section = False

            match = header_re.match(line)
            # Headers mark the start of a keyval section, but they do match
            # inside, so only match if outside a keyval section.
            if not in_keyval_section and match:
                in_keyval_section = True
                key, value = match.group('key', 'val')
                # Determine column for start of value - remember, this string
                # is now stripped so this may look different from your examples
                value_start_column = line.index(value)
                if key == "VG Name":
                    # New Volume Group - new vg_info_dict entry
                    self.vg_list.append({
                        # Logical and Physical volumes stored by name / device
                        'Logical Volumes': {},
                        'Physical Volumes': {},
                        'VG Name': value,
                    })
                    current_data_store = self.vg_list[-1]
                elif key == 'LV Path':
                    # New Logical Volume - append to 'Logical Volumes'
                    current_data_store = {
                        key: value,
                    }
                    self.vg_list[-1]['Logical Volumes'][value] = current_data_store
                else:  # elif key == 'PV Name':
                    # New Physical Volume - append to 'Physical Volumes'
                    current_data_store = {
                        key: value,
                    }
                    self.vg_list[-1]['Physical Volumes'][value] = current_data_store
            elif line == '':
                # Blank line = new section, stop expecting keys and values
                in_keyval_section = False
            elif in_keyval_section and len(line) > value_start_column:
                # Extract key and value by substring
                key = line[:value_start_column].strip()
                value = line[value_start_column:].strip()
                # Finally, add this key/val pair to the current data store.
                current_data_store[key] = value



"""
Custom datasources related to ``httpd``
"""
from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.combiners.ps import Ps
from insights.specs.datasources import get_running_commands


@datasource(Ps, HostContext)
def httpd_cmds(broker):
    """
    Function to search the output of ``ps auxcww`` to find all running Apache
    webserver processes and extract the binary path.

    Returns:
        list: List of the binary paths to each running process
    """
    cmds = get_running_commands(broker[Ps], broker[HostContext], ['httpd', ])
    if cmds:
        return cmds
    raise SkipComponent



import operator
import string
from functools import reduce
from insights.parsr import (
    EOF,
    EOL,
    InSet,
    LineEnd,
    Many,
    Number,
    OneLineComment,
    Opt,
    PosMarker,
    skip_none,
    String,
    WS,
    WSChar,
)
from insights.parsr.query import Entry

DATA = """
# this is a config file
a = 15
b = a string
valueless
d = 1.14

# another section
+valueless  # no value
e = hello   # a value
#
"""


def to_entry(ms):
    children = []
    for mark, value in ms:
        children.append(Entry(name=mark.value, attrs=[value], lineno=mark.lineno))
    return Entry(children=children)


class KVPairs:
    def __init__(self, sep_chars="=:", comment_chars="#;"):
        eol_chars = set("\n\r")
        sep_chars = set(sep_chars)
        comment_chars = set(comment_chars)
        key_chars = set(string.printable) - (sep_chars | eol_chars | comment_chars)
        value_chars = set(string.printable) - (eol_chars | comment_chars)

        OLC = reduce(operator.__or__, [OneLineComment(c) for c in comment_chars])
        Comment = (WS >> OLC).map(lambda x: None)
        Num = Number & (WSChar | LineEnd)
        Key = WS >> PosMarker(String(key_chars).map(str.strip)) << WS
        Sep = InSet(sep_chars)
        Value = WS >> (Num | String(value_chars).map(str.strip))
        KVPair = (Key + Opt(Sep + Value, default=[None, None])).map(
            lambda a: (a[0], a[1][1])
        )
        Line = Comment | KVPair | EOL.map(lambda x: None)
        Doc = Many(Line).map(skip_none).map(to_entry)
        self.Top = Doc + EOF

    def loads(self, s):
        return self.Top(s)[0]

    def load(self, f):
        return self.loads(f.read())


def test_marker():
    kvp = KVPairs()
    res = kvp.loads(DATA)
    assert res
    assert res["+valueless"][0].lineno == 9



from insights.parsers.mysql_log import MysqlLog
from insights.tests import context_wrap

MYSQL_LOG = """
2018-03-13T06:37:37.268209Z 0 [Warning] Changed limits: max_open_files: 1024 (requested 5000)
2018-03-13T06:37:37.268417Z 0 [Warning] Changed limits: table_open_cache: 431 (requested 2000)
2018-03-13T06:37:37.268549Z 0 [Warning] TIMESTAMP with implicit DEFAULT value is deprecated. Please use --explicit_defaults_for_timestamp server option (see documentation fo
r more details).
2018-03-13T06:37:39.651387Z 0 [Warning] InnoDB: New log files created, LSN=45790
2018-03-13T06:37:39.719166Z 0 [Warning] InnoDB: Creating foreign key constraint system tables.
2018-03-13T06:37:39.784406Z 0 [Warning] No existing UUID has been found, so we assume that this is the first time that this server has been started. Generating a new UUID: 0
698a7d6-2689-11e8-8944-0800274ac5ef.
2018-03-13T06:37:39.789636Z 0 [Warning] Gtid table is not ready to be used. Table 'mysql.gtid_executed' cannot be opened.
2018-03-13T06:37:40.498084Z 0 [Warning] CA certificate ca.pem is self signed.
2018-03-13T06:37:41.080591Z 1 [Warning] root@localhost is created with an empty password ! Please consider switching off the --initialize-insecure option.
md5_dgst.c(80): OpenSSL internal error, assertion failed: Digest MD5 forbidden in FIPS mode!
06:37:41 UTC - mysqld got signal 6 ;
2018-03-13T07:43:31.450772Z 0 [Note] Event Scheduler: Loaded 0 events
2018-03-13T07:43:31.450988Z 0 [Note] /opt/rh/rh-mysql57/root/usr/libexec/mysqld: ready for connections.
Version: '5.7.16'  socket: '/var/lib/mysql/mysql.sock'  port: 3306  MySQL Community Server (GPL)
md5_dgst.c(80): OpenSSL internal error, assertion failed: Digest MD5 forbidden in FIPS mode!
07:46:19 UTC - mysqld got signal 6 ;
"""


def test_mysql_log():
    log = MysqlLog(context_wrap(MYSQL_LOG))
    assert len(log.get("[Warning]")) == 9
    assert len(log.get("[Note]")) == 2
    assert 'ready for connections' in log



import doctest
import pytest
from insights.tests import context_wrap
from insights.parsers import ldif_config, SkipException
from insights.parsers.ldif_config import LDIFParser

LDIF_CONFIG = """
dn:
aci: (targetattr != "aci")(version 3.0; aci "rootdse anon read access"; allow(
 read,search,compare) userdn="ldap:///anyone";)
createTimestamp: 20201026161200Z
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=Directory Manager
modifyTimestamp: 20210608144722Z
nsslapd-return-default-opattr: namingContexts
nsslapd-return-default-opattr: supportedControl
nsslapd-return-default-opattr: supportedExtension
nsslapd-return-default-opattr: supportedLDAPVersion
nsslapd-return-default-opattr: supportedSASLMechanisms
nsslapd-return-default-opattr: vendorName
nsslapd-return-default-opattr: vendorVersion
objectClass: top

dn: cn=config
aci: (targetattr != aci)(version 3.0; aci "cert manager read access"; allow (r
 ead, search, compare) userdn = "ldap:///uid=pkidbuser,ou=people,o=ipaca";)
aci: (target = "ldap:///cn=automember rebuild membership,cn=tasks,cn=config")(
 targetattr=*)(version 3.0;acl "permission:Add Automember Rebuil Membership T
 ask";allow (add) groupdn = "ldap:///cn=Add Automember Rebuild Membership Task
 ,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)
aci: (targetattr = "cn || createtimestamp || entryusn || modifytimestamp || ob
 jectclass || passsyncmanagersdns*")(target = "ldap:///cn=ipa_pwd_extop,cn=plu
 gins,cn=config")(version 3.0;acl "permission:Read PassSync Managers Configura
 tion";allow (compare,read,search) groupdn = "ldap:///cn=Read PassSync Manager
 s Configuration,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)
cn: config1111
cn: config2222
cn1:: IGJlZ2lucyB3aXRoIGEgc3BhY2U=
cn2:< file:///tmp/value
modifiersName: cn=directory manager
modifyTimestamp: 20210609192548Z
objectClass: nsslapdConfig
CACertExtractFile: /etc/dirsrv/slapd-IDM-NYPD-FINEST/IDM.NYPD.FINEST20IPA20CA.
 pem
numSubordinates: 14
nsslapd-errorlog-level: 81920
nsslapd-rootpw: {SSHA512}mdrYu17fr9ukhID6B6/aMHE1KeMhWLwVfP3y2LSNtTFaMkRPf340X
 MGEN/ocUoAyykmDSMxVcF3ajVR3+f5mqmNqxUek9PYT

dn: cn=monitor
aci: (target ="ldap:///cn=monitor*")(targetattr != "aci || connection")(versio
 n 3.0; acl "monitor"; allow( read, search, compare ) userdn = "ldap:///anyone
 ";)
cn: monitor
createTimestamp: 20201026161200Z
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=server,cn=plugins,cn=config
modifyTimestamp: 20201026161200Z
objectClass: top
objectClass: extensibleObject
numSubordinates: 3

dn: cn=changelog5,cn=config
cn: changelog5
createTimestamp: 20201026161228Z
creatorsName: cn=Directory Manager
modifiersName: cn=Directory Manager
modifyTimestamp: 20201026161228Z
nsslapd-changelogdir: /var/lib/dirsrv/slapd-IDM-NYPD-FINEST/cldb
nsslapd-changelogmaxage: 7d
objectClass: top
objectClass: extensibleobject

dn: cn=encryption,cn=config
aci: (target ="ldap:///cn=monitor*")(targetattr != "aci)
CACertExtractFile: /etc/dirsrv/slapd-IDM-NYPD-FINEST/IDM.NYPD.FINEST20IPA20CA.
 pem
# ldif config
allowWeakCipher: off
cn: encryption
createTimestamp: 20201026161200Z
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=server,cn=plugins,cn=config
modifyTimestamp: 20201026161252Z
nsSSL3Ciphers: default
nsSSLClientAuth: allowed
nsSSLSessionTimeout: 0
objectClass: top
objectClass: nsEncryptionConfig
sslVersionMin: TLS1.0
numSubordinates: 1

dn: cn=features,cn=config
cn: features
objectClass: top
objectClass: nsContainer
numSubordinates: 5

dn: cn=mapping tree,cn=config
aci: (targetattr = "cn || createtimestamp || description || entryusn || modify
 timestamp || nsds50ruv || nsds5beginreplicarefresh || nsds5debugreplicatimeou
 t || nsds5flags || nsds5replicaabortcleanruv || nsds5replicaautoreferral || n
 sds5replicabackoffmax || nsds5replicabackoffmin || nsds5replicabinddn || nsds
 5replicabindmethod || nsds5replicabusywaittime || nsds5replicachangecount ||
 nsds5replicachangessentsincestartup || nsds5replicacleanruv || nsds5replicacl
 eanruvnotified || nsds5replicacredentials || nsds5replicaenabled || nsds5repl
 icahost || nsds5replicaid || nsds5replicalastinitend || nsds5replicalastinits
 tart || nsds5replicalastinitstatus || nsds5replicalastupdateend || nsds5repli
 calastupdatestart || nsds5replicalastupdatestatus || nsds5replicalegacyconsum
 er || nsds5replicaname || nsds5replicaport || nsds5replicaprotocoltimeout ||
 nsds5replicapurgedelay || nsds5replicareferral || nsds5replicaroot || nsds5re
 plicasessionpausetime || nsds5replicastripattrs || nsds5replicatedattributeli
 st || nsds5replicatedattributelisttotal || nsds5replicatimeout || nsds5replic
 atombstonepurgeinterval || nsds5replicatransportinfo || nsds5replicatype || n
 sds5replicaupdateinprogress || nsds5replicaupdateschedule || nsds5task || nsd
 s7directoryreplicasubtree || nsds7dirsynccookie || nsds7newwingroupsyncenable
 d || nsds7newwinusersyncenabled || nsds7windowsdomain || nsds7windowsreplicas
 ubtree || nsruvreplicalastmodified || nsstate || objectclass || onewaysync ||
  winsyncdirectoryfilter || winsyncinterval || winsyncmoveaction || winsyncsub
 treepair || winsyncwindowsfilter")(targetfilter = "(|(objectclass=nsds5Replic
 a)(objectclass=nsds5replicationagreement)(objectclass=nsDSWindowsReplicationA
 greement)(objectClass=nsMappingTree))")(version 3.0;acl "permission:Read Repl
 ication Agreements";allow (compare,read,search) groupdn = "ldap:///cn=Read Re
 plication Agreements,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)
aci: (targetattr=*)(version 3.0;acl "permission:Add Replication Agreements";al
 low (add) groupdn = "ldap:///cn=Add Replication Agreements,cn=permissions,cn=
 pbac,dc=idm,dc=nypd,dc=finest";)
aci: (targetattr=*)(targetfilter="(|(objectclass=nsds5Replica)(objectclass=nsd
 s5replicationagreement)(objectclass=nsDSWindowsReplicationAgreement)(objectCl
 ass=nsMappingTree))")(version 3.0; acl "permission:Modify Replication Agreeme
 nts"; allow (read, write, search) groupdn = "ldap:///cn=Modify Replication Ag
 reements,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)
aci: (targetattr=*)(targetfilter="(|(objectclass=nsds5replicationagreement)(ob
 jectclass=nsDSWindowsReplicationAgreement))")(version 3.0;acl "permission:Rem
 ove Replication Agreements";allow (delete) groupdn = "ldap:///cn=Remove Repli
 cation Agreements,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)
aci: (target = "ldap:///cn=meTo($dn),cn=*,cn=mapping tree,cn=config")(targetat
 tr = "objectclass || cn")(version 3.0; acl "Allow hosts to read their replica
 tion agreements"; allow(read, search, compare) userdn = "ldap:///fqdn=($dn),c
 n=computers,cn=accounts,dc=idm,dc=nypd,dc=finest";)
cn: mapping tree
modifiersName: cn=Directory Manager
modifyTimestamp: 20201026161600Z
objectClass: top
objectClass: extensibleObject
numSubordinates: 3

dn: cn=sasl,cn=config
cn: sasl
objectClass: top
objectClass: nsContainer
numSubordinates: 1
"""

LDIF_CONFIG_DOC = """
dn:
aci: (targetattr != "aci")(version 3.0; aci "rootdse anon read access"; allow(
 read,search,compare) userdn="ldap:///anyone";)
aci: (target = "ldap:///cn=automember rebuild membership,cn=tasks,cn=config")(
 ,cn=permissions,cn=pbac,dc=idm";)
createTimestamp: 20201026161200Z
creatorsName: cn=server,cn=plugins,cn=config
modifiersName: cn=Directory Manager
modifyTimestamp: 20210608144722Z
nsslapd-return-default-opattr: namingContexts
nsslapd-return-default-opattr: supportedControl
nsslapd-return-default-opattr: supportedExtension
nsslapd-return-default-opattr: supportedLDAPVersion
nsslapd-return-default-opattr: supportedSASLMechanisms
nsslapd-return-default-opattr: vendorName
nsslapd-return-default-opattr: vendorVersion
objectClass: top

dn: cn=changelog5,cn=config
cn: changelog5
createTimestamp: 20201026161228Z
creatorsName: cn=Directory Manager
modifiersName: cn=Directory Manager
modifyTimestamp: 20201026161228Z
nsslapd-changelogdir: /var/lib/dirsrv/slapd-IDM-NYPD-FINEST/cldb
nsslapd-changelogmaxage: 7d
objectClass: top
objectClass: extensibleobject
"""

LDIF_CONFIG_EMPTY = ""


def test_ldif_parser():
    ldif_config = LDIFParser(context_wrap(LDIF_CONFIG))
    for item in ldif_config:
        if item['dn'] == "":
            item['aci'] == '(targetattr != "aci")(version 3.0; aci "rootdse anon read access"; allow(read,search,compare) userdn="ldap:///anyone";)'
            item['modifiersName'] == 'cn=Directory Manager'
        if item['dn'] == 'cn=changelog5,cn=config':
            assert item['createTimestamp'] == '20201026161228Z'
            assert item['creatorsName'] == 'cn=Directory Manager'
            assert item['nsslapd-changelogdir'] == '/var/lib/dirsrv/slapd-IDM-NYPD-FINEST/cldb'
            assert item['objectClass'] == 'extensibleobject'
        if item['dn'] == 'cn=config':
            assert item['aci'] == '(targetattr != aci)(version 3.0; aci "cert manager read access"; allow (read, search, compare) userdn = "ldap:///uid=pkidbuser,ou=people,o=ipaca";)(target = "ldap:///cn=automember rebuild membership,cn=tasks,cn=config")(targetattr=*)(version 3.0;acl "permission:Add Automember Rebuil Membership Task";allow (add) groupdn = "ldap:///cn=Add Automember Rebuild Membership Task,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)(targetattr = "cn || createtimestamp || entryusn || modifytimestamp || objectclass || passsyncmanagersdns*")(target = "ldap:///cn=ipa_pwd_extop,cn=plugins,cn=config")(version 3.0;acl "permission:Read PassSync Managers Configuration";allow (compare,read,search) groupdn = "ldap:///cn=Read PassSync Managers Configuration,cn=permissions,cn=pbac,dc=idm,dc=nypd,dc=finest";)'
            assert item['modifiersName'] == 'cn=directory manager'
            assert item['modifyTimestamp'] == '20210609192548Z'
            assert item['objectClass'] == 'nsslapdConfig'
        if item['dn'] == 'cn=monitor':
            assert item['aci'] == '(target ="ldap:///cn=monitor*")(targetattr != "aci || connection")(version 3.0; acl "monitor"; allow( read, search, compare ) userdn = "ldap:///anyone";)'

    assert ldif_config.search(dn='cn=features,cn=config')[0] == ldif_config[5]
    assert ldif_config.search(dn='cn=sasl,cn=config')[0] == ldif_config[7]
    assert ldif_config.search(cn='features')[0] == ldif_config[5]
    assert ldif_config.search(dn='cn=sasl,cn=config')[0] == ldif_config[-1]


def test_empty():
    with pytest.raises(SkipException) as e:
        LDIFParser(context_wrap(LDIF_CONFIG_EMPTY))
    assert 'The file is empty' in str(e)


def test_ldif_config_doc_examples():
    env = {
        'ldif_config': LDIFParser(context_wrap(LDIF_CONFIG_DOC)),
    }
    failed, total = doctest.testmod(ldif_config, globs=env)
    assert failed == 0



import doctest
from insights.parsers.installed_product_ids import InstalledProductIDs
from insights.parsers import installed_product_ids
from insights.tests import context_wrap


COMMAND_OUTPUT = """
+-------------------------------------------+
Product Certificate
+-------------------------------------------+

Certificate:
    Path: /etc/pki/product-default/69.pem
    Version: 1.0
    Serial: 12750047592154749739
    Start Date: 2017-06-28 18:05:10+00:00
    End Date: 2037-06-23 18:05:10+00:00

Subject:
    CN: Red Hat Product ID [4f9995e0-8dc4-4b4f-acfe-4ef1264b94f3]

Issuer:
    C: US
    CN: Red Hat Entitlement Product Authority
    O: Red Hat, Inc.
    OU: Red Hat Network
    ST: North Carolina
    emailAddress: ca-support@redhat.com

Product:
    ID: 69
    Name: Red Hat Enterprise Linux Server
    Version: 7.4
    Arch: x86_64
    Tags: rhel-7,rhel-7-server
    Brand Type:
    Brand Name:


+-------------------------------------------+
    Product Certificate
+-------------------------------------------+

Certificate:
    Path: /etc/pki/product/69.pem
    Version: 1.0
    Serial: 12750047592154751271
    Start Date: 2018-04-13 11:23:50+00:00
    End Date: 2038-04-08 11:23:50+00:00

Subject:
    CN: Red Hat Product ID [f3c92a95-26be-4bdf-800f-02c044503896]

Issuer:
    C: US
    CN: Red Hat Entitlement Product Authority
    O: Red Hat, Inc.
    OU: Red Hat Network
    ST: North Carolina
    emailAddress: ca-support@redhat.com

Product:
    ID: 69
    Name: Red Hat Enterprise Linux Server
    Version: 7.6
    Arch: x86_64
    Tags: rhel-7,rhel-7-server
    Brand Type:
    Brand Name:
"""

NONE_FOUND = """
find: '/etc/pki/product-default/': No such file or directory
find: '/etc/pki/product/': No such file or directory
"""

BAD_FILE = """
Unable to read certificate file '/etc/pki/product/some_file.pem': Error loading certificate
"""


def test_installed_product_ids():
    results = InstalledProductIDs(context_wrap(COMMAND_OUTPUT))
    assert results is not None
    assert results.ids == set(['69', '69'])

    results = InstalledProductIDs(context_wrap(NONE_FOUND))
    assert results is not None
    assert results.ids == set([])

    results = InstalledProductIDs(context_wrap(BAD_FILE))
    assert results is not None
    assert results.ids == set([])


def test_installed_product_ids_docs():
    env = {
        'products': InstalledProductIDs(context_wrap(COMMAND_OUTPUT)),
    }
    failed, total = doctest.testmod(installed_product_ids, globs=env)
    assert failed == 0
